GADGET = {};


/**
 * Returns the ajax requests that should be executed prior to calling GADGET.template
 */
GADGET.templateArgs = function(baseUrl) {
	return [
	    {
	    	key: "versionInfo",
	    	ajaxOptions: function () {
	    		var gadget = this;
	    		var projectId = gadgets.util.unescapeString(gadget.getPref("projectId"));
	    		var versionId = gadgets.util.unescapeString(gadget.getPref("version"));
	    	    return {
	    	        url: baseUrl + "/rest/greenhopper/1.0/days-remaining/getVersionInfo",
	    	        data: {
	    	            projectId : projectId,
	    	            versionId : versionId
	    	        }
	    	    };
        	}
	    }
	];
};


/**
 * Rendering function for the days remaining gadget
 */
GADGET.template = function (gadget, args, baseUrl) {
	var project = args.versionInfo.project;
	var version = args.versionInfo.version;
	
	// empty content and set root classes
	gadget.getView().empty();
	var body = AJS.$('body');
	body.addClass('days-remaining-gadget');
	if (GH.gadget.utils.getViewName() == 'wallboard') body.addClass('wallboard');
	
	/**
	 * Renders an error message
	 */
	var showErrorMessage = function(message) {
		gadget.getView().append(AJS.$('<div class="gg-error-message">'+message+'</div>'));
	};
	
	// check whether we got a version
	if (! version || ! version.id) { // with auto we get back a partial version object instead of no version object!
		showErrorMessage(gadget.getMsg('gh.gadget.days.remaining.noReleaseDatesWarning'));
		return;
	} else if (! version.isDisplayable) {
		showErrorMessage(gadget.getMsg('gh.gadget.days.remaining.noReleaseDateText'));
		return;
	}
	
	// root element
	var $ggWrapper = AJS.$('<div/>').attr('class', 'gg-wrapper');
	gadget.getView().append($ggWrapper);
	
	// title with project and version links
	var $headerHtml = GH.gadget.utils.addProjectVersionHeader(gadget, args.versionInfo, $ggWrapper);
	
	// days remaining text
	function getDaysRemainingText(_version, strong) {
		var strongBegin = strong ? '<strong>' : '';
		var strongEnd = strong ? '</strong>' : '';
		if (_version.daysRemaining == 0 && ! _version.isOverdue) {
			return gadget.getMsg('gh.gadget.days.remaining.today');
		}
		else if (_version.daysRemaining == 1) {
			return strongBegin + gadget.getMsg('gh.gadget.days.remaining.day') + strongEnd + ' ' + gadget.getMsg('gh.gadget.days.remaining.remaining');
		}
		else {
			// display days remaining for daysRemaining > 1 or isOverdue
			return strongBegin + gadget.getMsg('gh.gadget.days.remaining.days') + strongEnd + ' ' + gadget.getMsg('gh.gadget.days.remaining.remaining');
		}
	};
	
	function getDaysRemainingClass(_version) {
		if (_version.isOverdue) {
			return 'gg-overdue';
		} else if (_version.daysRemaining == 0) {
			return 'gg-today';
		} else if (! _version.isDisplayable) {
			return 'gg-nodate';
		} else {
			return'gg-future';
		}
	}
	
	// main part
	var mainClass = getDaysRemainingClass(version);
	var daysValue = version.daysRemaining;
	daysText = getDaysRemainingText(version, true);
	var mainHtml = ''
		+ '<div class="gg-days-box '+mainClass+'">'
		+ '	 <div class="gg-days-value">'+daysValue+'</div>'
		+ '	 <div class="gg-days-text">'+daysText+'</div>'
		+ '</div>';
	AJS.$(mainHtml).appendTo($ggWrapper);
	
	// render parent version?
	if (gadget.getPref("showMasterOrChild") == "true" && version.masterVersion != null) {
		var parentVersion = version.masterVersion;
		var parentDetailsTitle = gadget.getMsg('gh.gadget.days.remaining.masterVersion');
		var parentVersionLink = GH.gadget.utils.getVersionLink(gadget, project.key, parentVersion.id);
		var releaseDateText = gadget.getMsg("gh.gadget.days.remaining.endDate") + " : " + parentVersion.releaseDate;
		var ggDaysClass = getDaysRemainingClass(parentVersion);
		
		if (parentVersion.isDisplayable) {
			var ggDaysText = parentVersion.daysRemaining + ' ' + getDaysRemainingText(parentVersion);
		} else {
			var ggDaysText = gadget.getMsg('gh.gadget.days.remaining.noReleaseDateText');
		}
		
		var parentHtml = ''
		+ '  <div class="gg-days-parent">'
		+ '	   <h6>'+parentDetailsTitle+'</h6>'
		+ '	   <div><a href="'+GH.gadget.utils.getProjectLink(gadget, project.key)+'" target="_parent">'+project.name+' ('+project.key+')</a> : <a href="'+parentVersionLink+'" target="_parent">'+parentVersion.name+'</a></div>'
		+ '	   <div>'+releaseDateText+'</div>'
		+ '    <div class="gg-days-text '+ggDaysClass+'">'+ggDaysText+'</div>'
		+ '  </div>'
		AJS.$(parentHtml).appendTo($ggWrapper);
	}
};

/**
 * Returns the ajax requests that should be executed prior to calling GADGET.descriptor.
 * The descriptor will get the results in the args parameter, with each request response added to the "key" field
 */
GADGET.descriptorArgs = function (baseUrl) {
	return [
        {
            key: "projectOptions",
            ajaxOptions:  baseUrl + "/rest/greenhopper/1.0/project-list"
        }
    ];
};

/**
 * Configuration screen descriptor for the days remaining gadget
 */ 
GADGET.descriptor = function (gadget, args, baseUrl) {
    return {
        theme : function()
        {
            if (gadgets.window.getViewportDimensions().width < 450)
            {
                return "gdt top-label";
            }
            else
            {
                return "gdt";
            }
        }(),
        fields: [
			GH.gadget.fields.projectPicker(gadget, "projectId", args.projectOptions.options),
			GH.gadget.fields.versionPicker(gadget, "projectId", "version", false, true),
			GH.gadget.fields.checkbox(gadget, "showMasterOrChild", 'gh.gadget.common.options.parentdetails', gadget.getPref('showMasterOrChild') == 'true'),
			GH.gadget.fields.checkbox(gadget, "showProjectName", 'gh.gadget.common.options.projectname', gadget.getPref('showProjectName') == 'true'),
	        GH.gadget.fields.checkbox(gadget, "showVersionName", 'gh.gadget.common.options.versionname', gadget.getPref('showVersionName') == 'true'),
            AJS.gadget.fields.nowConfigured()
        ]
    };
};
