AJS.$(function () {

    var helpDialog = new AJS.FormPopup({
        id: "keyboard-shortcuts-dialog",
        trigger: "#keyshortscuthelp",
        widthClass: "large",
        onContentRefresh: function () {
            var context = this.get$popupContent();
            AJS.$("a.submit-link", context).click(function(e){
                e.preventDefault();
                AJS.$("form", context).submit();
            });
        }
    });

    if (document.getElementById("dashboard")) {

        var deleteDashboardDialog = new AJS.FormPopup({type: "ajax"});
        /*
          Hijacking delete dashboard link
         */
        AJS.$(document).delegate("#delete_dashboard", "click", function(e) {
            e.stopPropagation();
            e.preventDefault();

            deleteDashboardDialog.$activeTrigger = AJS.$("#delete_dashboard");
            deleteDashboardDialog.init({
                type: "ajax",
                id: "delete-dshboard",
                ajaxOptions: { url: deleteDashboardDialog.$activeTrigger.attr("href") },
                targetUrl: "input[name=targetUrl]"
            });
            deleteDashboardDialog.show();
        });


        // Issue action dialogs should not be effective inside gadgets.
        return;
    }

	/* dialogs.js runs first, which will already initialize JIRA.DIALOGS.
	 * as part of the initialization setTransitions is called, so we end up having the in-page link urls hooked up to
	 * the trigger events already.
	 * Make sure we clear out the attached click handlers before adding our own ones
	 */
	(function() {
		var name, value;
		for (name in JIRA.DIALOGS) {
			value = JIRA.DIALOGS[name];
			if (value._unassignEvents && value.options.trigger) {
				// unassign the live events previously assigned by each dialog
				value._unassignEvents("trigger", value.options.trigger);
	    	}
		}
		// unassign the workflow dialog creation function
		AJS.$(document).undelegate("a.issueaction-workflow-transition", "click");
		
		// remove the attach screenshot live handler
		AJS.$("a.issueaction-attach-screenshot").die("click");
	})();	

	
	/**
	 * GreenHopper extensions
	 */
	AJS.$.namespace('GH');
	
	/**
	 * Holds the allowed resolutions for a given workflow transition action
	 * key: action id, value: object with name/value pairs
	 * 
	 * This field is set by execute-option.vm and task-options.vm and cleared
	 * by issuesactionspopup.js. If no entry for a given transition is found
	 * then all resolutions are assumed to be valid 
	 */
	if (! GH) GH = {};
	GH.allowedWorkflowResolutions = [];
	GH.setAllowedWorkflowResolutions = function(action, resolutions) {
		GH.allowedWorkflowResolutions[action] = resolutions;
	}
	GH.clearAllowedWorkflowResolutions = function() {
		GH.allowedWorkflowResolutions = [];
	}

	/**
     * We have to extend the FormPopup, but also the LabelsPopup. So lets first gather all we need and then extend both
     */
    GH.formPopupExtensions = {
        _getDefaultOptions: function () {
        	return AJS.$.extend(this._super(), {
        		/**
                 * Original defined in formpopup.js
                 * The original version reloads the page, we though want to hide the dialog and update the ui instead
                 */
        		onDialogFinished: function() {
        			// Original code:
            		// Ignore the redirect (set by some actions, such as clone or delete), and the reload too.
            		//if (this._hasTargetUrl()) {
                    //    window.location.href = this._getTargetUrlValue();
                    //} else {
                    //    AJS.reloadViaWindowLocation(window.location.href);
                    //}
        			this.handleSuccess();
            	}
        	})
        },
        
        /**
         * Overwrite _performRedirect to call handleSuccess instead
         */
        _performRedirect: function (response) {
        	/* orig code:
        	 * this.hide();
             * this.redirected = true;
             * this._super(url);
             */
        	this.redirected = true;
        	this.handleSuccess();
        },
        
        /** Called when the dialog initial request response is "serverIsDone". */
        _handleInitialDoneResponse: function (data, xhr, smartAjaxResult) {
        	// Remove our wait dialog
        	this._hideloadingIndicator();
        	this._super(data, xhr, smartAjaxResult);
        },
        
        /**
         * Overwrite hide() to call doCancelCleanup
         * 
         * handleCancel() isn't called when the cancel link is clicked, only when esc is used!
         * By overwriting hide we are at least sure we get always called  
         */
        hide: function (undim, success) {
        	// don't do anything if we are not the current popup
        	if (AJS.FlexiPopup.current !== this) {
                return false;
            }
        	
        	// first execute parent's hide function
        	this._super(undim);
        	
        	// callback in case this is a cancel hide
        	if (! success) {
        		this.doCancelCleanup();
        	}
        },
        
        /** Called when the form submitted successfully and either
         *  - _performRedirect or
         *  - onDialogFinished (which also performs a redirect or a window reload)
         */
        handleSuccess: function() {
        	// hide the dialog
    		this.hide(true, true);
    		
    		// get the issue we modified
    		var issue = Boards.firstSelectedIssue();
        	
        	// update the ui, which is action dependent
        	var formId = this.options.id || '';
        	if (formId.startsWith('workflow-transition-')) {
        		formId = 'workflow-transition-';
        	}
        	
            switch (formId) {
        	// issue refresh cases
        	case 'comment-add-dialog':
        	case 'attach-file-dialog':
        	case 'link-issue-dialog':
        	case 'assign-to-me-link-handler':
        		if(Boards.inSearchMode){
        			Boards.needsRefresh = true;
        			issue.refresh();
        			Boards.refreshSearchStats(AJS.$('#searchKey').val(), AJS.$('#searchType').val());
        			Boards.returnToSearch();
        		} else {
        			issue.refresh();
        		}
        		gh.app.message.showIssueMessage(issue.id);
        		break;
        		
        	// board reload cases
        	default:
        		if(Boards.inSearchMode){
        			// retrieve the search board again
        			Boards.needsRefresh = true;
        			Boards.retrieve(AJS.$('#searchKey').val(), AJS.$('#searchType').val(), Boards.type(), issue.id);
        			gh.app.message.showIssueMessage(issue.id);
        		}
        		else if (Boards.isPlanningBoard()) {
			 		Boards.mainBoard.refreshMainBoard(Boards.mainBoard.getStart(), [issue.key]);
			 		Boards.refreshColumn();
			 		gh.app.message.showIssueMessage(issue.id);
        		}
        		else { // Task Board
        			switch(formId) {
        			case 'delete-issue-dialog':
        				// issue key done, just refresh the board
        				Boards.submit(Boards.type(true));
        				break;
        			case 'workflow-transition-':
        				// if we are on the outline view we need to check whether we are a parent issue - in which case we only have to update the issue and the status bar
        				// This here is quite a hack - we're looking at the triangle subtask expander icon to figure out if we're actually a parent issue (transitioning with
        				// the link on the right side of the outline view). The visiblity check has to be done since the expander icon is hidden when subtasks are not shown
        				// using the context.
        				// TODO un-bork
        				var collapse = AJS.$('#colapse_'+issue.key);
        				if ((! Boards.isCompactView()) && collapse.length > 0 && GH.Util.isVisible(collapse[0])) {
        					Boards.updatePBar();
        					issue.refresh();
        					gh.app.message.showIssueMessage(issue.id);
        				} else {
        					// refresh the dragged from and dragged to boards, but only if a dragging took place
        					// if a user uses the operations dialog to do a workflow transition then we have to refresh
        					// the whole page as we cannot tell where the issue will have landed
        					if (Boards.droppedBoard) {
        						Boards.refreshStepBoardsForTransition(issue);
        						delete Boards.droppedBoard;
        					} else {
        						// refresh the board and try to select the issue
                				Boards.getBoardForIssue(issue.key, Boards.type());
        					}
        				}
        			    break;
        			default:
        				// refresh the board and try to select the issue
        				Boards.getBoardForIssue(issue.key, Boards.type());
        				break;
        			}
        		}
        		break;
        	}
        },
        
        /**
         * Called when the form has been canceled.
         */
        doCancelCleanup: function() {            	
        	// see whether there is any cleanup work to be done
        	var formId = this.options.id || '';
        	if (formId.startsWith('workflow-transition-')) {
        		formId = 'workflow-transition-';
        	}
            switch (formId) {
            case 'workflow-transition-':
            	if (! Boards.isPlanningBoard()) {
            		var issue = Boards.firstSelectedIssue();
            		if (issue) {
            			issue.board.cancel([issue.key]);
            		}
            	}
            	delete Boards.droppedBoard;
            	break;
            }
            
            // make sure we return to the search dialog if previously displayed
            if(Boards.inSearchMode){
            	Boards.returnToSearch();
    		}
        },

        show: function () {
        	this._super();
        },
        
        /**
         * Make sure we use our waiting display instead of the one provided by popup.js
         */
        _showloadingIndicator: function() {
        	// dim and display the waiting spinner
        	Boards.dimAndMoveDialogElements();
        	Boards.hideSearchPopup();
        	GH.Util.hideAll(['confirmation']);
        	GH.Util.showAll(['dialog', 'waiting_display']);
        },
        
        /**
         * Hide our waiting display, not the one provided by popup.js
         */
        _hideloadingIndicator: function() {
        	// remove the waiting spinner, but leave the page dimmed.
        	// either dialog closing, page refresh or board update will take care of taking the dim away
        	GH.Util.hideAll(['dialog', 'waiting_display']);
        }
    };
    
	/**
	 * Extend FormPopup, as we need to overwrite one of its functions
	 */
	GH.GhFormPopup = AJS.FormPopup.extend(GH.formPopupExtensions);
	
    /**
     * Following object handles the allowedTransitions hack we need in order to
     * support task board columns with only a subset of all possible resolutions.
     * Preferably try adding stuff to GhFormPopup!
     */
    GH.GhWorkflowFormPopup = GH.GhFormPopup.extend({
    	/**
    	 * Overwrite to add a callback for content refresh
    	 */
    	init: function(options) {
    		this._super(options);
    		this.onContentReady(this.setAllowedResolutions);
    	},
    
    	/**
    	 * Adapts the resolutions dropdown to only display resolutions defined in the target column
    	 */
    	setAllowedResolutions: function() {
    		// remove resolutions from dropdown
    		var resolutions = GH.allowedWorkflowResolutions[this.options.transitionActionId]
    		if (resolutions) {
    			//var allowed = this.options.allowedTransitions;
    			$popup = this.get$popup();
    			var resOptions = $popup.find("select[name=resolution]").find("option");
    			resOptions.each(function(index, elem) {
    				var $elem = AJS.$(elem); 
    				if (! resolutions[$elem.attr('value')]) {
    					$elem.remove();
    				}
    			});
    		}
    	}
    });
    
    /**
     * Some actions we want to hijack, to directly hook them up with our own javascript code.
     * Note that by using this handler NO remote call using the orignal url is performed.
     */
    GH.LinkHandler = GH.GhFormPopup.extend({
        _getDefaultOptions: function () {
        	return AJS.$.extend(this._super(), {
        		linkHandler: function() {}
        	});
    	},
    	
    	show: function() {
            var myEvent = new AJS.$.Event("beforeShow");
            
            if (AJS.FlexiPopup.current) {
                AJS.FlexiPopup.current.hide(false);
                delete(AJS.FlexiPopup.current);
                AJS.undim();
            }
            
            this.options.linkHandler.call(this)
    	}
    });
    
    /**
     * Some actions redirect to another jira page. Try to show them in an iframe instead
     */
    GH.IFrameHandler = GH.LinkHandler.extend({
    	_getDefaultOptions: function () {
        	return AJS.$.extend(this._super(), {
        		// use the url provided, either show in iframe or redirect the browser
        		linkHandler: function() {
        			var issueId = jira.app.issuenavigator.getSelectedIssueId();
        			var ops = this._getAjaxOptionsObject();
        			if (issueId && ops && ops.url) {
        				Boards.jiraIssueAction(issueId, ops.url);
        			}
        		}
        	});
    	}
    });

    /**
     * The labels popup is a bit a special case, in that it implements some custom stuff on top of what formpopup provides.
     * First add our formPopupExtensions, then handle the additional stuff
     * 
     * In short, the popup in-document replaces the labels instead of triggering a complete page refresh. That's what the
     * labelsProvider is used for. All magic is actually happening inside LabelsPopup.js/_handleSubmitResponse.
     * 
     * Solution to this problem: Simply re-overwrite the function with the default implementation of FormPopup.
     */
    GH.LabelsPopup = AJS.LabelsPopup.extend(GH.formPopupExtensions);
    GH.LabelsPopup = GH.LabelsPopup.extend({
    	_handleSubmitResponse: function (data, xhr, smartAjaxResult) {
    		if (this.serverIsDone) {
                if (this.options.onSuccessfulSubmit) {
                    this.options.onSuccessfulSubmit.call(this, data, xhr, smartAjaxResult);
                }
                if (this.options.autoClose) {
                    this.hide();
                }
                if (this.options.onDialogFinished) {
                    this.options.onDialogFinished.call(this, data, xhr, smartAjaxResult);
                }
            }
    	},
    	
    	/**
    	 * This function ensures that the keyboard shortcut hint is not displayed for custom fields
    	 */
    	decorateContent: function() {
    		this._super();
    		
	        var $content = this.get$popupContent();
	        var $customFieldId = $content.find('input[name=customFieldId]');
	        if ($customFieldId.length === 1) {
	        	var $hint = $content.find('#hint');
	        	$hint.hide();
	        }
    	}
    });
	
    JIRA.DIALOGS = {
    	// make sure we use the iframe for the edit
    	editIssue: new GH.IFrameHandler({
    		id: "edit-issue-iframe-handler",
    		trigger: "a.issueaction-edit-issue",
    		ajaxOptions: getAjaxOptions
    	}),
    	
    	// display voters in an iframe
    	viewVoters: new GH.IFrameHandler({
    		id: "view-voters-iframe-handler",
    		trigger: "a.issueaction-view-voters",
    		ajaxOptions: getAjaxOptions
    	}),
    	
    	// watchers too
    	manageWatchers: new GH.IFrameHandler({
    		id: "manage-watchers-iframe-handler",
    		trigger: "a.issueaction-manage-watchers",
    		ajaxOptions: getAjaxOptions
    	}),
    	
    	// By default the create issue page is displayed. Show our create issue dialog instead
    	createSubtask: new GH.LinkHandler({
    		id: "create-subtask-link-handler",
    		trigger: "a.issueaction-create-subtask",
    		ajaxOptions: getAjaxOptions,
    		linkHandler: function() {
    			var issueKey = jira.app.issuenavigator.getSelectedIssueKey();
    			Boards.newSubTask(issueKey, false);
    		}
    	}),
    	
    	// assign to me - the default reloads the page (as it is a normal link)
    	assignToMe: new GH.LinkHandler({
    		id: "assign-to-me-link-handler",
    		trigger: "a.issueaction-assign-to-me",
    		ajaxOptions: getAjaxOptions,
    		onSuccessfulSubmit : storeCurrentIssueIdOnSucessfulSubmit,
    		issueMsg : 'thanks_issue_assigned',
    		linkHandler: function() {
    			var issue = Boards.firstSelectedIssue();
    			issue.updateField('assignee', GH.CONFIG.currentUser);
    			this.options.onSuccessfulSubmit.call(this);
    			gh.app.message.showIssueMessage(issue.id);
    		}
    	}),

    	
        linkIssue: new GH.GhFormPopup({
            id: "link-issue-dialog",
            trigger: "a.issueaction-link-issue",
            ajaxOptions: getAjaxOptions,
            onSuccessfulSubmit : storeCurrentIssueIdOnSucessfulSubmit,
            issueMsg : 'thanks_issue_linked',
            onContentRefresh: function () {
                var context = this.get$popupContent(),
                    issuePicker = new AJS.IssuePicker({
                        element: AJS.$("#linkKey", context),
                        userEnteredOptionsMsg: AJS.params.enterIssueKey,
                        uppercaseUserEnteredOnSelect: true
                    });

                jQuery(".overflow-ellipsis").textOverflow();
                applyCommentControls(context);
            }
        }),

        deleteIssue: new GH.GhFormPopup({
            id: "delete-issue-dialog",
            trigger: "a.issueaction-delete-issue",
            targetUrl: "#delete-issue-return-url",
            ajaxOptions: getAjaxOptions,
            onSuccessfulSubmit : storeCurrentIssueIdOnSucessfulSubmit,
            issueMsg : 'thanks_issue_deleted',
            onContentRefresh: function () {
                jQuery(".overflow-ellipsis").textOverflow();
            }
        }),

        cloneIssue: new GH.GhFormPopup({
            id: "clone-issue-dialog",
            trigger: "a.issueaction-clone-issue",
            handleRedirect:true,
            ajaxOptions: getAjaxOptions,
            onSuccessfulSubmit : storeCurrentIssueIdOnSucessfulSubmit,
            issueMsg : 'thanks_issue_cloned',
            onContentRefresh: function () {
                jQuery(".overflow-ellipsis").textOverflow();
                applyCommentControls(this.get$popupContent());
            }
        }),

        assignIssue: new GH.GhFormPopup({
            id: "assign-dialog",
            trigger: "a.issueaction-assign-issue",
            ajaxOptions: getAjaxOptions,
            onSuccessfulSubmit : storeCurrentIssueIdOnSucessfulSubmit,
            issueMsg : 'thanks_issue_assigned',
            onContentRefresh: function () {
                jQuery(".overflow-ellipsis").textOverflow();
                var context = this.get$popupContent();
                applyCommentControls(context);
                jira.app.issueoperations.wireAssignToMeLink(context);
            }
        }),

        logWork: new GH.GhFormPopup({
            id: "log-work-dialog",
            trigger: "a.issueaction-log-work",
            handleRedirect:true,
            ajaxOptions: getAjaxOptions,
            onSuccessfulSubmit : storeCurrentIssueIdOnSucessfulSubmit,
            issueMsg : 'thanks_issue_worklogged',
            onContentRefresh: function () {
                jQuery(".overflow-ellipsis").textOverflow();
                var context = this.get$popupContent(),
                    logWorkSelector = '#log-work-adjust-estimate-new-value, #log-work-adjust-estimate-manual-value';
                applyCommentControls(context);
                applyLogworkControls(context);
            }
        }),

        attachFile: new GH.GhFormPopup ({
            id: "attach-file-dialog",
            trigger: "a.issueaction-attach-file",
            handleRedirect: true,
            ajaxOptions: getAjaxOptions,
            onSuccessfulSubmit : storeCurrentIssueIdOnSucessfulSubmit,
            issueMsg : 'thanks_issue_attached',
            onContentRefresh: function () {
                jQuery(".overflow-ellipsis").textOverflow();
                var context = this.get$popupContent();
                applyCommentControls(context);
            }
        }),

        attachScreenshot: new jira.app.attachments.screenshot.ScreenshotWindow ({
            id: "attach-screenshot-window",
            trigger: "a.issueaction-attach-screenshot"
        }),

        comment: new GH.GhFormPopup({
            id: "comment-add-dialog",
            trigger: ":not(.ops) > * > a.issueaction-comment-issue",
            handleRedirect: true,
            ajaxOptions: getAjaxOptions,
            onSuccessfulSubmit : storeCurrentIssueIdOnSucessfulSubmit,
            issueMsg : 'thanks_issue_commented',
            onContentRefresh: function () {
                jQuery(".overflow-ellipsis").textOverflow();
                applyCommentControls(this.get$popupContent());
            }
        }),

        editLabels: new GH.LabelsPopup({
            id: "edit-labels-dialog",
            trigger: "a.issueaction-edit-labels,a.edit-labels",
            autoClose: true,
            ajaxOptions: getAjaxOptions,
            onSuccessfulSubmit : storeCurrentIssueIdOnSucessfulSubmit,
            issueMsg : 'thanks_issue_labelled',
            labelsProvider: labelsProvider,
            onContentRefresh: function () {
                jQuery(".overflow-ellipsis").textOverflow();
            }
        }),
        
        keyboardShortcuts: helpDialog 
    };

	/*
    function getAjaxOptions () {
        var $focusRow = jira.app.issuenavigator.get$focusedRow();
        var linkIssueURI = this.options.url || this.$activeTrigger.attr("href");

        if (/id=\{0\}/.test(linkIssueURI)) {
            if (!$focusRow.length) {
                return false;
            } else {
                linkIssueURI = linkIssueURI.replace(/(id=\{0\})/, "id=" + $focusRow.attr("rel"));
            }
        }

        if (jira.app.issuenavigator.isNavigator()) {
            var result = /[?&]id=([0-9]+)/.exec(linkIssueURI);
            this.issueId = result && result.length == 2 ? result[1] : null;
            if(this.issueId !== $focusRow.attr("rel")) {
                //if the issue id doesn't match the focused row's issue id then reassign focus and get the
                //issuekey from the newly focused row! This can happen when clicking the pencil for the
                //labels picker.
                jira.app.issuenavigator.shortcuts.focusRow(this.issueId);
                $focusRow = jira.app.issuenavigator.get$focusedRow();
            }
            this.issueKey = jira.app.issuenavigator.getSelectedIssueKey();
        }

        return {
            data: {decorator: "dialog", inline: "true"},
            url: linkIssueURI
        };
    }
    */
	
	/**
	 * GH Version of getAjaxOptions.
	 * 
	 * issueId: comes from getSelectedIssueId(), which we overwrite in gh-issueactionspopup.js.
		 *   We always select an issue before performing an action (be it the cog menu, the dot menu or a drag and drop operation), so it is save to
		 *   simply fetch the selected issue.
	 * linkIssueURI: either comes from the dot dialog (complete url in this case) or from the hidden links, where {0} is used for the issue id
	 */
	function getAjaxOptions() {
        var issueId = jira.app.issuenavigator.getSelectedIssueId();
        var linkIssueURI = this.options.url || this.$activeTrigger.attr("href");

        // replace {0} if present
        if (/id=\{0\}/.test(linkIssueURI)) {
            if (! issueId) {
                return false;
            } else {
                linkIssueURI = linkIssueURI.replace(/(id=\{0\})/, "id=" + issueId);
            }
        }
		
        // now return all requested data
		return {
            data: {decorator: "dialog", inline: "true"},
            url: linkIssueURI
        };
	}

    /**
     * Stores the current issue id into session storage if the dialogs submits successfully
     */
    function storeCurrentIssueIdOnSucessfulSubmit() {
        //if (jira.app.issuenavigator.isNavigator()) {
            var issueId = this.issueId;
            var issueKey = this.issueKey;
            // always fetch the values from the helper fnctions
            //if (! issueId) {
                issueId = jira.app.issuenavigator.getSelectedIssueId();
                issueKey = jira.app.issuenavigator.getSelectedIssueKey();
            //}
            if (issueId)
            {
                var sessionStorge = jira.app.session.storage;
                sessionStorge.setItem('selectedIssueId', issueId);
                sessionStorge.setItem('selectedIssueKey', issueKey);
                sessionStorge.setItem('selectedIssueMsg', this.options.issueMsg);
            }
        //}
        this.issueId = null;
        this.issueKey = null;
    }


    function applyCommentControls(context) {
        new AJS.SecurityLevelMenu(AJS.$("#commentLevel", context));
        var wikiRenders = jQuery(".wiki-js-prefs", context);
        wikiRenders.each(function() {
            jira.app.wikiPreview(this, context).init();
        });
    }

    function applyLogworkControls(context){
        jQuery('#log-work-adjust-estimate-new-value, #log-work-adjust-estimate-manual-value', context).attr('disabled','disabled');
        jQuery('#log-work-adjust-estimate-'+jQuery('input[name=worklog_adjustEstimate]:checked,input[name=adjustEstimate]:checked', context).val()+'-value', context).removeAttr('disabled');
        jQuery('input[name=worklog_adjustEstimate],input[name=adjustEstimate]', context).change(function(){
            jQuery('#log-work-adjust-estimate-new-value,#log-work-adjust-estimate-manual-value', context).attr('disabled','disabled');
            jQuery('#log-work-adjust-estimate-'+jQuery(this).val()+'-value', context).removeAttr('disabled');
        });

        AJS.$(function () {
            AJS.$(context).find("#log-work-activate").change(function() {
                AJS.$(context).find("#worklog-logworkcontainer").toggleClass("hidden");
                if (AJS.$(context).find("#worklog-timetrackingcontainer").size() > 0) {
                    AJS.$(context).find("#worklog-timetrackingcontainer").toggleClass("hidden");
                }
            });
        });
    }

    /*
    function labelsProvider(labelsPopup) {
        var $trigger = labelsPopup.$activeTrigger,
            $labelsContainer = $trigger.closest(".labels-wrap"),
            isSubtaskForm = $trigger.parents("#view-subtasks").length !== 0;

        if(isSubtaskForm) {
            //if we clicked the subtask form, only look within the current row for a labels wrap!
            $labelsContainer = $trigger.parents("tr").find(".labels-wrap");
        } else if ($trigger.hasClass("issueaction-edit-labels")) {
            // we clicked the issueaction which should only update the system field labels!
            if (jira.app.issuenavigator.isNavigator()) {
                $labelsContainer = jQuery("#issuetable tr.issuerow.focused td.labels .labels-wrap");
            } else {
                $labelsContainer = jQuery("#wrap-labels .labels-wrap");
            }
        }

        if($labelsContainer.length > 0) {
            return $labelsContainer;
        }
        return false;
    }
    */
    function labelsProvider(labelsPopup) {}
        
    
    /**
     * Dialogs should only show up if there's an issue to work on!
     */
    AJS.$.each(JIRA.DIALOGS, function (name, dialog) {
        if (dialog instanceof AJS.FlexiPopup) {
            AJS.$(dialog).bind("beforeShow", function (e) {
                //for all dialogs except for the keyboard shortcuts, check if we have a valid issue to work on!
                if(this != JIRA.DIALOGS.keyboardShortcuts) {
                    return jira.app.issuenavigator.isRowSelected() || jira.app.issue.getIssueId() !== undefined;
                }
            });
        }
    });
    
    // Workflow transition dialogs
    AJS.$(document).delegate("a.issueaction-workflow-transition", "click", function(event) {
        event.preventDefault();
        var action = /action=(\d+)/.exec(this.href.slice(this.href.indexOf("?")));
        if (action) {
            var id = "workflow-transition-" + action[1] + "-dialog";
            var $trigger = AJS.$(this);
            if (!JIRA.DIALOGS[id]) {
                JIRA.DIALOGS[id] = new GH.GhWorkflowFormPopup({
                    id: id,
                    url: $trigger.attr("href"),
                    // GreenHopper: this id is read just before the dialog is displayed and used to fetch the resolutions that should be active
                    transitionActionId: action[1],
                    trigger: 'a[href*="' + action[0] + '"].issueaction-workflow-transition',
                    widthClass: "large",
                    handleRedirect: true,
                    ajaxOptions: getAjaxOptions,
                    onSuccessfulSubmit : storeCurrentIssueIdOnSucessfulSubmit,
                    issueMsg : 'thanks_issue_transitioned',
                    onContentRefresh: function () {
                        var context = this.get$popupContent();
                        // initialise AJS tabs for the workflow dialogs
                        AJS.tabs.setup();
                        applyCommentControls(context);
                        jira.app.issueoperations.wireAssignToMeLink(context);
                        applyLogworkControls(context);
                    }
                });
                JIRA.DIALOGS[id].show();
            }
        }
    });
});