(function($) {

    /**
     * Holds global GreenHopper keyboard shortcut implementation code 
     *
     * @class gh.app.globalkeyboardshortcuts
     * @requires AJS
     */
    $.namespace("gh.app.globalkeyboardshortcuts");
    var shortcuts = gh.app.globalkeyboardshortcuts;

    /**
     * Get the selected issue id.
     * 
     * This function checks for functions available on the issuenavigator and issue pages to fetch the id.
     * 
     * GreenHopper provides overwrites for all these functions, so this code will work on GreenHopper pages too.
     */
    shortcuts.findSelectedIssueId = function() {
    	// fetch the current issue id, either from the navigator or the issue page
    	var issueId;
    	if (jira && jira.app && jira.app.issuenavigator && jira.app.issuenavigator.getSelectedIssueId) {
    		issueId = jira.app.issuenavigator.getSelectedIssueId();
    	}
    	if (!issueId && jira && jira.app && jira.app.issue && jira.app.issue.getIssueId) {
    		issueId = jira.app.issue.getIssueId();
    	}
    	return issueId;
    };
    
    /**
     * Implementation of the "go to planning board" shortcut.
     */
    shortcuts.goToPlanningBoard = function() {
    	shortcuts.goToBoard("VB", '#planningboard-mlink');
    };
    
    /**
     * Implementation of the "go to task board" shortcut.
     */
    shortcuts.goToTaskBoard = function() {
    	shortcuts.goToBoard("TB", '#taskboard-mlink');
    };
    
    /**
     * Goes to a board, given the board type and the linkSelector to use in case no issue is found.
     */
    shortcuts.goToBoard = function(boardType, linkSelector) {
    	var issueId = shortcuts.findSelectedIssueId();
    	if (issueId && contextPath) {
    		var url = contextPath+'/secure/GetBoardForIssue.jspa?type='+boardType+'&id='+issueId;
    		AJS.reloadViaWindowLocation(url);
    	} else {
    		// just navigate to the planning board
    		shortcuts.followLink(linkSelector);
    	}
    };
   
    /**
     * Follows a link given a link selector.
     */
    shortcuts.followLink = function(selector) {
    	// Copied from whenitype.js
    	var elem = $(selector);
        if (elem.length > 0 &&
                (elem.attr("nodeName").toLowerCase() === "a" || elem.attr("nodeName").toLowerCase() === "link")) {
            elem.click();
            window.location.href = elem.attr("href");
        }
    };

})(AJS.$);