//
// We have to completely redefine issueactionspopup as we want to use a different return url for the actions...
// unfortunately that's hidden deep inside a couple of closures
//

jQuery.namespace("jira.app.issueActionsPopup");


AJS.IssueActionsPopup = AJS.FlexiPopup.extend({


    _getDefaultOptions: function () {
        return AJS.$.extend(this._super(), {
            cached: false,
            id: "issue-actions-dialog",
            widthClass: "small"
        });
    },

    _setContent: function (content, decorate) {
        if (content) {
            this._super(content, decorate);
        } else {
            this._super(AJS.$([
                "<form id='issue-actions-dialog-form' class='aui'>",
                    "<div class='content-body'>",
                        "<div id='issueactions-suggestions' class='aui-list' />",
                        "<div class='description'>",
                        AJS.params.issueActionsHint,
                        "</div>",
                    "</div>",
                "</form>"
            ].join('')), true);
        }
        if (AJS.FlexiPopup.current === this) {
            var triggerConfig = new jira.setFocus.FocusConfiguration();
            triggerConfig.context = this.get$popup()[0];
            triggerConfig.parentElementSelectors = ['.content-body'];
            jira.setFocus.pushConfiguration(triggerConfig);
            jira.setFocus.triggerFocus();
        }
    },

    _formatActionsResponse: function (response) {

        function addSelected(issueId) {

             var url = window.location.href,
                newUrl = url;

            if (/selectedIssueId=[0-9]*/.test(url)) {
                newUrl = newUrl.replace(/selectedIssueId=[0-9]*/g, "selectedIssueId=" + issueId);
            } else {
                if (jira.app.issuenavigator.isNavigator()) {
                    if (/\?/.test(url)) {
                        newUrl = newUrl + "&";
                    } else {
                        newUrl = newUrl + "?";          
                    }
                    newUrl = newUrl + "selectedIssueId=" + issueId;
                }
            }
            return encodeURIComponent(newUrl);
        }

        function addSelectedGH(issueId) {
        	// we want to load getBoardForIssue on action successes
        	// Overwrite the type for the taskboard, as SB (searchboard) would refer to the planningboard
        	var boardType = Boards.type(false, true);
        	var boardId = Boards.mainBoard.id;
        	var newUrl = '/secure/GetBoardForIssue.jspa?type='+boardType+'&selectedBoardId='+boardId+'&id='+issueId;
        	return encodeURIComponent(newUrl);
        }
        
        function formatWorkflowResponse(workflowResponse) {

            var workflows = new AJS.GroupDescriptor({
                label: AJS.params.workflow
            });

            AJS.$(workflowResponse).each(function() {

            	// GreenHopper: make sure the issue id is not hardcoded, thus inserted by getAjaxOptions.
                workflows.addItem(new AJS.ItemDescriptor({
                    href: contextPath + "/secure/WorkflowUIDispatcher.jspa?id={0}&action=" + this.action + "&atl_token=" + response.atlToken + "&returnUrl=" + addSelectedGH(response.id),
                    label: this.name,
                    styleClass: "issueaction-workflow-transition"
                }));

            });

            return workflows;
        }

        function formatOperationResonse(operationsResponse) {

            var operations = new AJS.GroupDescriptor({
                label: AJS.params.actions
            });

            AJS.$(operationsResponse).each(function() {

                var _returnUrl;
/*
                if (this.name === "Clone") {
                    if (jira.app.issuenavigator.isNavigator()) {
                          _returnUrl=addSelected(response.id);
                    } else {
                          _returnUrl="";
                    }
                } else {
*/                    _returnUrl=addSelectedGH(response.id);
/*                }
*/

                operations.addItem(new AJS.ItemDescriptor({
                    href: this.url + "&returnUrl=" + _returnUrl,
                    label: this.name,
                    styleClass: this.styleClass
                }));
            });
            return operations;
        }

        var res = [];

        if (response) {
            if (response.actions && response.actions.length != 0) {
                res.push(formatWorkflowResponse(response.actions));
            }

            if (response.operations && response.operations.length != 0) {
                res.push(formatOperationResonse(response.operations));
            }

        }

        return res;
    },

    decorateContent: function () {

        var instance = this,
            issueKey = jira.app.issuenavigator.getSelectedIssueKey(),
            issueId = jira.app.issue.getIssueId() || jira.app.issuenavigator.getSelectedIssueId();

        if (issueKey) {
            this.addHeading(AJS.params.dotOperations + ": <span>" + issueKey + "</span>");
        } else {
            this.addHeading(AJS.params.dotOperations);
        }

        this.queryControl = new AJS.QueryableDropdown({
            id: "issueactions",
            element: this.$content.find("#issueactions-suggestions"),
            ajaxOptions: {
                minQueryLength: 1,
                dataType: "json",
                url: AJS.format(contextPath + "/rest/api/1.0/issues/{0}/ActionsAndOperations?atl_token={1}", issueId, atl_token()),
                formatResponse: this._formatActionsResponse
            },
            showDropdownButton: true,
            loadOnInit: true
        });
        this.queryControl._handleServerError  = function(smartAjaxResult) {
            var errMsg = jira.ajax.buildSimpleErrorContent(smartAjaxResult);
            var errorClass = smartAjaxResult.status === 401?'warn':'error';
            instance._setContent(AJS.$('<div class="ajaxerror"><div class="notify ' + errorClass+'"><p>' + errMsg + '</p></div></div>'), false);
            instance._addCloseLink();
        };


        this.timeoutId = undefined;
        this._addCloseLink();

    },

    _addCloseLink : function() {
        var instance=this,$closeLink, $buttonContainer, $buttons;
        
        $buttonContainer = AJS.$('<div class="buttons-container content-footer"></div>').appendTo(this.get$popupContent());
        $buttons = AJS.$('<div class="buttons"/>').appendTo($buttonContainer);
        $closeLink = AJS.$("<a href='#' class='cancel' id='aui-dialog-close'>" + AJS.params.closelink + "</a>");
        $closeLink.appendTo($buttons, this.get$popupContent()).click(function(e) {
            instance.hide();
            e.preventDefault();
        });
        this.get$popupContent().append($buttonContainer);
    },

    hide: function (undim) {

        if (this._super(undim) === false) {
            return false;
        }

        jira.setFocus.popConfiguration();        
    }
});

jira.app.issueActionsPopup = new AJS.IssueActionsPopup();

//make sure we only show the dialog if we have a selected issue.
AJS.$(jira.app.issueActionsPopup).bind("beforeShow", function (e) {
    return jira.app.issuenavigator.isRowSelected() || jira.app.issue.getIssueId() !== undefined;
});

/*
 * 
 * New code. attach a couple of onContentReady handlers as well as redefine some global getIssueId/getIssueKey functions
 * Also defines new message functions plus handles the "enter" key for issue viewing
 * 
 */
AJS.$(function () {
	
	/**
	 * Issue actions support
	 */
	jQuery.namespace("gh.app.issueActionsPopup");
	
	// our keyboard shortcuts accessed the issue actions popup to be located in following variable
	gh.app.issueActionsPopup = jira.app.issueActionsPopup;
	
	/* This function wires the close button/link up to the handleCancel function
	 * (which is by default only called when closing the popup via escape).
	 */
	var wireCloseLinkToHandleCancel = function() {
		var $closeLink = this.get$popupContent().find('a.cancel');
		// remove original click handler
		$closeLink.unbind('click');
		var popup = this;
		// and bind a slightly adapted one
		$closeLink.click(function(e){
		    popup.handleCancel.call(popup);
		    e.preventDefault();
		});
	};
	/* This function temporarily hides the search screen if it is displayed. */
	var hideSearchDialog = function() {
		if(Boards.inSearchMode){
			Boards.hideSearchPopup();
		}
	}
	// Attach both function to the popup, they get executed right before the dialog is displayed
	gh.app.issueActionsPopup.onContentReady(hideSearchDialog);
	gh.app.issueActionsPopup.onContentReady(wireCloseLinkToHandleCancel);
	// also clear the workflow resolutions
	gh.app.issueActionsPopup.onContentReady(GH.clearAllowedWorkflowResolutions);
	
	// overwrite what we do when handleCancel is called
	gh.app.issueActionsPopup.handleCancel=function(){
		if (AJS.FlexiPopup.current === this) {
			this.hide();
			if(Boards.inSearchMode){
	        	Boards.returnToSearch();
			}
		}
	}
	
	
	// Issueactionspopup.js uses following lines to find the key/id of an issue:
	//    issueKey = jira.app.issuenavigator.getSelectedIssueKey(),
	//    issueId = jira.app.issue.getIssueId() || jira.app.issuenavigator.getSelectedIssueId();
	// so simply make sure these methods return what is expected
	jira.app.issuenavigator.getSelectedIssueKey = function() {
		var issue = Boards.firstSelectedIssue();
		if (issue) {
			return issue.key;
		}
	};
	jira.app.issuenavigator.getSelectedIssueId = function() {
		var issue = Boards.firstSelectedIssue();
		if (issue) {
			return issue.id;
		}
	}
	jira.app.issue.getIssueId = jira.app.issuenavigator.getSelectedIssueId;
	
	
	/**
	 * Update messages support
	 */
	jQuery.namespace("gh.app.message");
	
	// Adaption of issuenavigator-shortcuts.js/flashIssueRow()
	gh.app.message.showIssueMessage = function(issueId) {
	    var sessionstorage = jira.app.session.storage;
	    var selectedIssueMsg = null;
	    var selectedIssueKey = null;
	    if (! issueId)
	    {
	        if (! issueId)
	        {
	            // ok try session storage next
	            issueId = sessionstorage.getItem('selectedIssueId');
	        }
	    }
	    if (issueId)
	    {
	        selectedIssueKey = sessionstorage.getItem('selectedIssueKey');
	        selectedIssueMsg = sessionstorage.getItem('selectedIssueMsg');
	        gh.app.message.showIssueMessageImpl(issueId, selectedIssueMsg, selectedIssueKey);
	    }
	    // always remove the state.  We want to be empty until some other action decides it wants to record this
	    sessionstorage.removeItem('selectedIssueId');
	    sessionstorage.removeItem('selectedIssueKey');
	    sessionstorage.removeItem('selectedIssueMsg');
	};
	
	// Adaption of issuenavigator-shortcuts.js/flashIssueRowWithId
	gh.app.message.showIssueMessageImpl = function(issueId, selectedIssueMsg, selectedIssueKey) {
	    // if we don't have an message then use a generic one if possible
	    if (! selectedIssueMsg)
	    {
	        selectedIssueMsg = 'thanks_issue_updated';
	    }
	
	    var msgText = AJS.params[selectedIssueMsg];
	    if (msgText && selectedIssueKey)
	    {
	        msgText = AJS.format(msgText, selectedIssueKey);
	        Boards.showMessage(msgText);
	    }
	}
	
	// Register a document ready handler to show any not yet displayed message
	AJS.$(document).ready(function () {
		gh.app.message.showIssueMessage();
	});
	
	//This is basically a hard coded shortcut for "Enter".  It will trigger a issue view
	AJS.$(document).keypress(function(e){
	    if (e.keyCode == '13'){
	        var target = e.target;
	        // On different browser the originalTarget is different, but all of these are impossible for a user to trigger.
	        if (target === undefined || target.nodeName === "HTML" || target.nodeName === "BODY" || target == document){
	        	Boards.viewSelectedIssue();
	        }
	    }
	});
});