// script.aculo.us effects.js v1.8.3, Thu Oct 08 11:23:33 +0200 2009

// Copyright (c) 2005-2009 Thomas Fuchs (http://script.aculo.us, http://mir.aculo.us)
// Contributors:
//  Justin Palmer (http://encytemedia.com/)
//  Mark Pilgrim (http://diveintomark.org/)
//  Martin Bialasinki
//
// script.aculo.us is freely distributable under the terms of an MIT-style license.
// For details, see the script.aculo.us web site: http://script.aculo.us/

// converts rgb() and #xxx to #xxxxxx format,
// returns self (or first argument) if not convertable
String.prototype.parseColor = function() {
  var color = '#';
  if (this.slice(0,4) == 'rgb(') {
    var cols = this.slice(4,this.length-1).split(',');
    var i=0; do { color += parseInt(cols[i]).toColorPart() } while (++i<3);
  } else {
    if (this.slice(0,1) == '#') {
      if (this.length==4) for(var i=1;i<4;i++) color += (this.charAt(i) + this.charAt(i)).toLowerCase();
      if (this.length==7) color = this.toLowerCase();
    }
  }
  return (color.length==7 ? color : (arguments[0] || this));
};

/*--------------------------------------------------------------------------*/

Element.collectTextNodes = function(element) {
  return $A($(element).childNodes).collect( function(node) {
    return (node.nodeType==3 ? node.nodeValue :
      (node.hasChildNodes() ? Element.collectTextNodes(node) : ''));
  }).flatten().join('');
};

Element.collectTextNodesIgnoreClass = function(element, className) {
  return $A($(element).childNodes).collect( function(node) {
    return (node.nodeType==3 ? node.nodeValue :
      ((node.hasChildNodes() && !Element.hasClassName(node,className)) ?
        Element.collectTextNodesIgnoreClass(node, className) : ''));
  }).flatten().join('');
};

Element.setContentZoom = function(element, percent) {
  element = $(element);
  element.setStyle({fontSize: (percent/100) + 'em'});
  if (Prototype.Browser.WebKit) window.scrollBy(0,0);
  return element;
};

Element.getInlineOpacity = function(element){
  return $(element).style.opacity || '';
};

Element.forceRerendering = function(element) {
  try {
    element = $(element);
    var n = document.createTextNode(' ');
    element.appendChild(n);
    element.removeChild(n);
  } catch(e) { }
};

/*--------------------------------------------------------------------------*/

var Effect = {
  _elementDoesNotExistError: {
    name: 'ElementDoesNotExistError',
    message: 'The specified DOM element does not exist, but is required for this effect to operate'
  },
  Transitions: {
    linear: Prototype.K,
    sinoidal: function(pos) {
      return (-Math.cos(pos*Math.PI)/2) + .5;
    },
    reverse: function(pos) {
      return 1-pos;
    },
    flicker: function(pos) {
      var pos = ((-Math.cos(pos*Math.PI)/4) + .75) + Math.random()/4;
      return pos > 1 ? 1 : pos;
    },
    wobble: function(pos) {
      return (-Math.cos(pos*Math.PI*(9*pos))/2) + .5;
    },
    pulse: function(pos, pulses) {
      return (-Math.cos((pos*((pulses||5)-.5)*2)*Math.PI)/2) + .5;
    },
    spring: function(pos) {
      return 1 - (Math.cos(pos * 4.5 * Math.PI) * Math.exp(-pos * 6));
    },
    none: function(pos) {
      return 0;
    },
    full: function(pos) {
      return 1;
    }
  },
  DefaultOptions: {
    duration:   1.0,   // seconds
    fps:        100,   // 100= assume 66fps max.
    sync:       false, // true for combining
    from:       0.0,
    to:         1.0,
    delay:      0.0,
    queue:      'parallel'
  },
  tagifyText: function(element) {
    var tagifyStyle = 'position:relative';
    if (Prototype.Browser.IE) tagifyStyle += ';zoom:1';

    element = $(element);
    $A(element.childNodes).each( function(child) {
      if (child.nodeType==3) {
        child.nodeValue.toArray().each( function(character) {
          element.insertBefore(
            new Element('span', {style: tagifyStyle}).update(
              character == ' ' ? String.fromCharCode(160) : character),
              child);
        });
        Element.remove(child);
      }
    });
  },
  multiple: function(element, effect) {
    var elements;
    if (((typeof element == 'object') ||
       Object.isFunction(element)) &&
    (element.length))
   elements = element;
 else
   elements = $(element).childNodes;

 var options = Object.extend({
   speed: 0.1,
   delay: 0.0
 }, arguments[2] || { });
 var masterDelay = options.delay;

 $A(elements).each( function(element, index) {
   new effect(element, Object.extend(options, { delay: index * options.speed + masterDelay }));
 });
},
PAIRS: {
 'slide':  ['SlideDown','SlideUp'],
 'blind':  ['BlindDown','BlindUp'],
 'appear': ['Appear','Fade']
},
toggle: function(element, effect, options) {
 element = $(element);
 effect  = (effect || 'appear').toLowerCase();
 
 return Effect[ Effect.PAIRS[ effect ][ element.visible() ? 1 : 0 ] ](element, Object.extend({
   queue: { position:'end', scope:(element.id || 'global'), limit: 1 }
 }, options || {}));
}
};

Effect.DefaultOptions.transition = Effect.Transitions.sinoidal;

/* ------------- core effects ------------- */

Effect.ScopedQueue = Class.create(Enumerable, {
initialize: function() {
 this.effects  = [];
 this.interval = null;
},
_each: function(iterator) {
 this.effects._each(iterator);
},
add: function(effect) {
 var timestamp = new Date().getTime();

 var position = Object.isString(effect.options.queue) ?
   effect.options.queue : effect.options.queue.position;

 switch(position) {
   case 'front':
     // move unstarted effects after this effect
     this.effects.findAll(function(e){ return e.state=='idle' }).each( function(e) {
         e.startOn  += effect.finishOn;
         e.finishOn += effect.finishOn;
       });
     break;
   case 'with-last':
     timestamp = this.effects.pluck('startOn').max() || timestamp;
     break;
   case 'end':
     // start effect after last queued effect has finished
     timestamp = this.effects.pluck('finishOn').max() || timestamp;
     break;
 }

 effect.startOn  += timestamp;
 effect.finishOn += timestamp;

 if (!effect.options.queue.limit || (this.effects.length < effect.options.queue.limit))
   this.effects.push(effect);

 if (!this.interval)
   this.interval = setInterval(this.loop.bind(this), 15);
},
remove: function(effect) {
 this.effects = this.effects.reject(function(e) { return e==effect });
 if (this.effects.length == 0) {
   clearInterval(this.interval);
   this.interval = null;
 }
},
loop: function() {
 var timePos = new Date().getTime();
 for(var i=0, len=this.effects.length;i<len;i++)
   this.effects[i] && this.effects[i].loop(timePos);
}
});

Effect.Queues = {
instances: $H(),
get: function(queueName) {
 if (!Object.isString(queueName)) return queueName;

 return this.instances.get(queueName) ||
   this.instances.set(queueName, new Effect.ScopedQueue());
}
};
Effect.Queue = Effect.Queues.get('global');

Effect.Base = Class.create({
position: null,
start: function(options) {
 if (options && options.transition === false) options.transition = Effect.Transitions.linear;
 this.options      = Object.extend(Object.extend({ },Effect.DefaultOptions), options || { });
 this.currentFrame = 0;
 this.state        = 'idle';
 this.startOn      = this.options.delay*1000;
 this.finishOn     = this.startOn+(this.options.duration*1000);
 this.fromToDelta  = this.options.to-this.options.from;
 this.totalTime    = this.finishOn-this.startOn;
 this.totalFrames  = this.options.fps*this.options.duration;

 this.render = (function() {
   function dispatch(effect, eventName) {
     if (effect.options[eventName + 'Internal'])
       effect.options[eventName + 'Internal'](effect);
     if (effect.options[eventName])
       effect.options[eventName](effect);
   }

   return function(pos) {
     if (this.state === "idle") {
       this.state = "running";
       dispatch(this, 'beforeSetup');
       if (this.setup) this.setup();
       dispatch(this, 'afterSetup');
     }
     if (this.state === "running") {
       pos = (this.options.transition(pos) * this.fromToDelta) + this.options.from;
       this.position = pos;
       dispatch(this, 'beforeUpdate');
       if (this.update) this.update(pos);
       dispatch(this, 'afterUpdate');
     }
   };
 })();

 this.event('beforeStart');
 if (!this.options.sync)
   Effect.Queues.get(Object.isString(this.options.queue) ?
     'global' : this.options.queue.scope).add(this);
},
loop: function(timePos) {
 if (timePos >= this.startOn) {
   if (timePos >= this.finishOn) {
     this.render(1.0);
     this.cancel();
     this.event('beforeFinish');
     if (this.finish) this.finish();
     this.event('afterFinish');
     return;
   }
   var pos   = (timePos - this.startOn) / this.totalTime,
       frame = (pos * this.totalFrames).round();
   if (frame > this.currentFrame) {
     this.render(pos);
     this.currentFrame = frame;
   }
 }
},
cancel: function() {
 if (!this.options.sync)
   Effect.Queues.get(Object.isString(this.options.queue) ?
     'global' : this.options.queue.scope).remove(this);
 this.state = 'finished';
},
event: function(eventName) {
 if (this.options[eventName + 'Internal']) this.options[eventName + 'Internal'](this);
 if (this.options[eventName]) this.options[eventName](this);
},
inspect: function() {
 var data = $H();
 for(property in this)
   if (!Object.isFunction(this[property])) data.set(property, this[property]);
 return '#<Effect:' + data.inspect() + ',options:' + $H(this.options).inspect() + '>';
}
});

Effect.Parallel = Class.create(Effect.Base, {
initialize: function(effects) {
 this.effects = effects || [];
 this.start(arguments[1]);
},
update: function(position) {
 this.effects.invoke('render', position);
},
finish: function(position) {
 this.effects.each( function(effect) {
   effect.render(1.0);
   effect.cancel();
   effect.event('beforeFinish');
   if (effect.finish) effect.finish(position);
   effect.event('afterFinish');
 });
}
});

Effect.Tween = Class.create(Effect.Base, {
initialize: function(object, from, to) {
 object = Object.isString(object) ? $(object) : object;
 var args = $A(arguments), method = args.last(),
   options = args.length == 5 ? args[3] : null;
 this.method = Object.isFunction(method) ? method.bind(object) :
   Object.isFunction(object[method]) ? object[method].bind(object) :
   function(value) { object[method] = value };
 this.start(Object.extend({ from: from, to: to }, options || { }));
},
update: function(position) {
 this.method(position);
}
});

Effect.Event = Class.create(Effect.Base, {
initialize: function() {
 this.start(Object.extend({ duration: 0 }, arguments[0] || { }));
},
update: Prototype.emptyFunction
});

Effect.Opacity = Class.create(Effect.Base, {
initialize: function(element) {
 this.element = $(element);
 if (!this.element) throw(Effect._elementDoesNotExistError);
 // make this work on IE on elements without 'layout'
 if (Prototype.Browser.IE && (!this.element.currentStyle.hasLayout))
   this.element.setStyle({zoom: 1});
 var options = Object.extend({
   from: this.element.getOpacity() || 0.0,
   to:   1.0
 }, arguments[1] || { });
 this.start(options);
},
update: function(position) {
 this.element.setOpacity(position);
}
});

Effect.Move = Class.create(Effect.Base, {
initialize: function(element) {
 this.element = $(element);
 if (!this.element) throw(Effect._elementDoesNotExistError);
 var options = Object.extend({
   x:    0,
   y:    0,
   mode: 'relative'
 }, arguments[1] || { });
 this.start(options);
},
setup: function() {
 this.element.makePositioned();
 this.originalLeft = parseFloat(this.element.getStyle('left') || '0');
 this.originalTop  = parseFloat(this.element.getStyle('top')  || '0');
 if (this.options.mode == 'absolute') {
   this.options.x = this.options.x - this.originalLeft;
   this.options.y = this.options.y - this.originalTop;
 }
},
update: function(position) {
 this.element.setStyle({
   left: (this.options.x  * position + this.originalLeft).round() + 'px',
   top:  (this.options.y  * position + this.originalTop).round()  + 'px'
 });
}
});

//for backwards compatibility
Effect.MoveBy = function(element, toTop, toLeft) {
return new Effect.Move(element,
 Object.extend({ x: toLeft, y: toTop }, arguments[3] || { }));
};

Effect.Scale = Class.create(Effect.Base, {
initialize: function(element, percent) {
 this.element = $(element);
 if (!this.element) throw(Effect._elementDoesNotExistError);
 var options = Object.extend({
   scaleX: true,
   scaleY: true,
   scaleContent: true,
   scaleFromCenter: false,
   scaleMode: 'box',        // 'box' or 'contents' or { } with provided values
   scaleFrom: 100.0,
   scaleTo:   percent
 }, arguments[2] || { });
 this.start(options);
},
setup: function() {
 this.restoreAfterFinish = this.options.restoreAfterFinish || false;
 this.elementPositioning = this.element.getStyle('position');

 this.originalStyle = { };
 ['top','left','width','height','fontSize'].each( function(k) {
   this.originalStyle[k] = this.element.style[k];
 }.bind(this));

 this.originalTop  = this.element.offsetTop;
 this.originalLeft = this.element.offsetLeft;

 var fontSize = this.element.getStyle('font-size') || '100%';
 ['em','px','%','pt'].each( function(fontSizeType) {
   if (fontSize.indexOf(fontSizeType)>0) {
     this.fontSize     = parseFloat(fontSize);
     this.fontSizeType = fontSizeType;
   }
 }.bind(this));

 this.factor = (this.options.scaleTo - this.options.scaleFrom)/100;

 this.dims = null;
 if (this.options.scaleMode=='box')
   this.dims = [this.element.offsetHeight, this.element.offsetWidth];
 if (/^content/.test(this.options.scaleMode))
   this.dims = [this.element.scrollHeight, this.element.scrollWidth];
 if (!this.dims)
   this.dims = [this.options.scaleMode.originalHeight,
                this.options.scaleMode.originalWidth];
},
update: function(position) {
 var currentScale = (this.options.scaleFrom/100.0) + (this.factor * position);
 if (this.options.scaleContent && this.fontSize)
   this.element.setStyle({fontSize: this.fontSize * currentScale + this.fontSizeType });
 this.setDimensions(this.dims[0] * currentScale, this.dims[1] * currentScale);
},
finish: function(position) {
 if (this.restoreAfterFinish) this.element.setStyle(this.originalStyle);
},
setDimensions: function(height, width) {
 var d = { };
 if (this.options.scaleX) d.width = width.round() + 'px';
 if (this.options.scaleY) d.height = height.round() + 'px';
 if (this.options.scaleFromCenter) {
   var topd  = (height - this.dims[0])/2;
   var leftd = (width  - this.dims[1])/2;
   if (this.elementPositioning == 'absolute') {
     if (this.options.scaleY) d.top = this.originalTop-topd + 'px';
     if (this.options.scaleX) d.left = this.originalLeft-leftd + 'px';
   } else {
     if (this.options.scaleY) d.top = -topd + 'px';
     if (this.options.scaleX) d.left = -leftd + 'px';
   }
 }
 this.element.setStyle(d);
}
});

Effect.Highlight = Class.create(Effect.Base, {
initialize: function(element) {
 this.element = $(element);
 if (!this.element) throw(Effect._elementDoesNotExistError);
 var options = Object.extend({ startcolor: '#ffff99' }, arguments[1] || { });
 this.start(options);
},
setup: function() {
 // Prevent executing on elements not in the layout flow
 if (this.element.getStyle('display')=='none') { this.cancel(); return; }
 // Disable background image during the effect
 this.oldStyle = { };
 if (!this.options.keepBackgroundImage) {
   this.oldStyle.backgroundImage = this.element.getStyle('background-image');
   this.element.setStyle({backgroundImage: 'none'});
 }
 if (!this.options.endcolor)
   this.options.endcolor = this.element.getStyle('background-color').parseColor('#ffffff');
 if (!this.options.restorecolor)
   this.options.restorecolor = this.element.getStyle('background-color');
 // init color calculations
 this._base  = $R(0,2).map(function(i){ return parseInt(this.options.startcolor.slice(i*2+1,i*2+3),16) }.bind(this));
 this._delta = $R(0,2).map(function(i){ return parseInt(this.options.endcolor.slice(i*2+1,i*2+3),16)-this._base[i] }.bind(this));
},
  update: function(position) {
    this.element.setStyle({backgroundColor: $R(0,2).inject('#',function(m,v,i){
      return m+((this._base[i]+(this._delta[i]*position)).round().toColorPart()); }.bind(this)) });
  },
  finish: function() {
    this.element.setStyle(Object.extend(this.oldStyle, {
      backgroundColor: this.options.restorecolor
    }));
  }
});

Effect.ScrollTo = function(element) {
  var options = arguments[1] || { },
  scrollOffsets = document.viewport.getScrollOffsets(),
  elementOffsets = $(element).cumulativeOffset();

  if (options.offset) elementOffsets[1] += options.offset;

  return new Effect.Tween(null,
    scrollOffsets.top,
    elementOffsets[1],
    options,
    function(p){ scrollTo(scrollOffsets.left, p.round()); }
  );
};

Element.CSS_PROPERTIES = $w(
  'backgroundColor backgroundPosition borderBottomColor borderBottomStyle ' +
  'borderBottomWidth borderLeftColor borderLeftStyle borderLeftWidth ' +
  'borderRightColor borderRightStyle borderRightWidth borderSpacing ' +
  'borderTopColor borderTopStyle borderTopWidth bottom clip color ' +
  'fontSize fontWeight height left letterSpacing lineHeight ' +
  'marginBottom marginLeft marginRight marginTop markerOffset maxHeight '+
  'maxWidth minHeight minWidth opacity outlineColor outlineOffset ' +
  'outlineWidth paddingBottom paddingLeft paddingRight paddingTop ' +
  'right textIndent top width wordSpacing zIndex');

Element.CSS_LENGTH = /^(([\+\-]?[0-9\.]+)(em|ex|px|in|cm|mm|pt|pc|\%))|0$/;

String.__parseStyleElement = document.createElement('div');
String.prototype.parseStyle = function(){
  var style, styleRules = $H();
  if (Prototype.Browser.WebKit)
    style = new Element('div',{style:this}).style;
  else {
    String.__parseStyleElement.innerHTML = '<div style="' + this + '"></div>';
    style = String.__parseStyleElement.childNodes[0].style;
  }

  Element.CSS_PROPERTIES.each(function(property){
    if (style[property]) styleRules.set(property, style[property]);
  });

  if (Prototype.Browser.IE && this.include('opacity'))
    styleRules.set('opacity', this.match(/opacity:\s*((?:0|1)?(?:\.\d*)?)/)[1]);

  return styleRules;
};

if (document.defaultView && document.defaultView.getComputedStyle) {
Element.getStyles = function(element) {
 var css = document.defaultView.getComputedStyle($(element), null);
 return Element.CSS_PROPERTIES.inject({ }, function(styles, property) {
   styles[property] = css[property];
   return styles;
 });
};
} else {
Element.getStyles = function(element) {
 element = $(element);
 var css = element.currentStyle, styles;
 styles = Element.CSS_PROPERTIES.inject({ }, function(results, property) {
   results[property] = css[property];
   return results;
 });
 if (!styles.opacity) styles.opacity = element.getOpacity();
 return styles;
};
}

Effect.Methods = {
morph: function(element, style) {
 element = $(element);
 new Effect.Morph(element, Object.extend({ style: style }, arguments[2] || { }));
 return element;
},
visualEffect: function(element, effect, options) {
 element = $(element);
 var s = effect.dasherize().camelize(), klass = s.charAt(0).toUpperCase() + s.substring(1);
 new Effect[klass](element, options);
 return element;
},
highlight: function(element, options) {
 element = $(element);
 new Effect.Highlight(element, options);
 return element;
}
};

$w('fade appear grow shrink fold blindUp blindDown slideUp slideDown '+
'pulsate shake puff squish switchOff dropOut').each(
function(effect) {
 Effect.Methods[effect] = function(element, options){
   element = $(element);
   Effect[effect.charAt(0).toUpperCase() + effect.substring(1)](element, options);
   return element;
 };
}
);

$w('getInlineOpacity forceRerendering setContentZoom collectTextNodes collectTextNodesIgnoreClass getStyles').each(
  function(f) { Effect.Methods[f] = Element[f]; }
);

Element.addMethods(Effect.Methods);